import {
  select,
  scaleLinear,
  scaleBand,
  axisLeft,
  axisRight,
  max,
} from 'd3';

import {
  format,
  extractContent,
} from './dataFormat';

import { artists } from './artists';

const svg = select('svg');

var margin = {
  left: 50,
  top: 10,
  bottom: 20,
  right: 10,
}; // the margins of the chart

const height =
  +svg.attr('height') -
  margin.top -
  margin.bottom;
const width =
  +svg.attr('width') - margin.left - margin.right;

svg
  .attr(
    'width',
    width + margin.left + margin.right
  )
  .attr(
    'height',
    height + margin.top + margin.bottom
  )
  .append('g')
  .attr(
    'transform',
    'translate(' +
      margin.left +
      ',' +
      margin.top +
      ')'
  );

var selectArtists = document.getElementById(
  'artists'
);

for (var i = 0; i < artists.length; i++) {
  var opt = artists[i];
  var el = document.createElement('option');
  el.textContent = opt;
  el.value = opt;
  selectArtists.appendChild(el);
}

function artistSelected() {
  // Create request object
  var request = new Request(
    'https://wasabi.i3s.unice.fr/api/v1/artist_all/name/' +
      this.value
  );
  fetch(request)
    .then((response) => response.json())
    .then((data) => {
      var songs = [];
      data.albums.forEach((d) => {
        d.songs.forEach((s) => {
          songs.push(extractContent(s.lyrics));
        });
      });
      update(songs);
    });
}

// Add X axis
var x = d3
  .scaleLinear()
  .range([margin.left, width]);
var xAxis = svg
  .append('g')
  .attr(
    'transform',
    'translate(0,' + height + ')'
  );

// Y axis
var y = d3
  .scaleBand()
  .range([margin.top, height - margin.top])
  .padding(1);
var yAxis = svg
  .append('g')
  .attr(
    'transform',
    'translate(' + margin.left + ',0)'
  );

function update(data) {
  data = format(data); // Go see dataFormat.js

  x.domain([0, max(data, (d) => +d.count)]);
  xAxis
    .transition()
    .duration(1000)
    .call(d3.axisBottom(x));

  y.domain(
    data.map(function (d) {
      return d.word;
    })
  );
  yAxis
    .transition()
    .duration(1000)
    .call(d3.axisLeft(y));

  // Lines
  var ln = svg.selectAll('.myLine').data(data);
  ln.enter()
    .append('line')
    .attr('class', 'myLine')
    .merge(ln)
    .transition()
    .duration(500)
    .attr('x2', function (d) {
      return x(d.count);
    })
    .attr('x1', x(0))
    .attr('y1', function (d) {
      return y(d.word);
    })
    .attr('y2', function (d) {
      return y(d.word);
    })
    .attr('stroke', 'black');

  // Circles
  var cl = svg.selectAll('circle').data(data);
  cl.enter()
    .append('circle')
    .merge(cl)
    .transition()
    .duration(500)
    .attr('r', '6')
    .style('fill', 'red')
    .attr('stroke', 'black')
    .attr('cx', function (d) {
      return x(d.count);
    })
    .attr('cy', function (d) {
      return y(d.word);
    });
}

selectArtists.addEventListener(
  'change',
  artistSelected,
  false
);

var request = new Request(
  'https://wasabi.i3s.unice.fr/api/v1/artist_all/name/Lara Fabian'
);

var songs = [];

fetch(request)
  .then((response) => response.json())
  .then((data) => {
  console.log(data);
    data.albums.forEach((d) => {
      d.songs.forEach((s) => {
        songs.push(extractContent(s.lyrics));
      });
    });
    update(songs);
  });
